module.exports = function(grunt) {

  var libs = ['bower_components/jsonlint/lib/jsonlint.js', 'bower_components/tv4/tv4.js', 'bower_components/jshint/dist/jshint.js'];

  var src = ['src/util/UtilDevTools.js',
             'src/developer/services/DTRenderService.js',
             'src/developer/services/SocketService.js',
             'src/**/*.js',
             '!src/developer/directives/*.js',
             'src/developer/directives/*.js',
             '!src/developer/directives/zhSchemaValidator.js',
             'src/developer/directives/zhSchemaValidator.js'];

  var testLibs = ["bower_components/angular-mocks/angular-mocks.js", "bower_components/q/q.js"];
  var helperPath = 'specs/helpers/**/*.js';
  var specsPath = 'assets/specs/**/*spec*.js';
  var mainZeedhi = 'bower_components/zeedhi-frontend/assets/dist/main.min.js';

  grunt.initConfig({
    bump: {
      options: {
        files: ['package.json', 'bower.json'],
        updateConfigs: [],
        commit: true,
        commitMessage: 'Release v%VERSION%',
        commitFiles: ['bower.json', 'package.json'],
        createTag: true,
        tagName: '%VERSION%',
        tagMessage: 'Version %VERSION%',
        push: true,
        pushTo: 'origin',
        gitDescribeOptions: '--tags --always --abbrev=1 --dirty=-d',
        globalReplace: false,
        prereleaseName: false,
        metadata: '',
        regExp: false
      }
    },
    uglify: {
      options: {
        mangle: false,
        compress: false,
        report: 'min',
        banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
      },
      dist: {
        files: {
          'dist/developer-enrichment.min.js': src.concat(libs)
        }
      }
    },
    concat: {
      dist: {
        src: src.concat(libs),
        dest: 'dist/developer-enrichment.js',
        options: {
          process: function(src, filepath) {
            return '\n' + '// FILE: ' + filepath + '\n' + src;
          }
        }
      }
    },
    watch: {
      pivotal: {
        files: src.concat(libs),
        tasks: ['jshint', 'concat', 'uglify']
      }
    },
    jshint: {
      all: ['Gruntfile.js', src]
    },
    jasmine: {
      pivotal: {
        src: [src],
        options: {
          keepRunner: true,
          vendor: [mainZeedhi, testLibs],
          specs: specsPath,
          helpers: helperPath,
          template: require('grunt-template-jasmine-istanbul'),
          templateOptions: {
            coverage: 'bin/coverage/coverage.json',
            report: 'bin/coverage',
            thresholds: {
              lines: 1.0,
              statements: 1.0,
              branches: 1.0,
              functions: 1.0
            }
          }
        }
      },
      toCommit: {
        src: [src],
        options: {
          vendor: [mainZeedhi, testLibs],
          specs: specsPath,
          helpers: helperPath
        }
      }
    },
  });

  grunt.loadNpmTasks('grunt-bump');
  grunt.loadNpmTasks('grunt-contrib-jshint');
  grunt.loadNpmTasks('grunt-contrib-concat');
  grunt.loadNpmTasks('grunt-contrib-uglify');
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.loadNpmTasks('grunt-contrib-jasmine');

  grunt.registerTask('default', ['jshint', 'uglify', 'jasmine:pivotal', 'concat']);
  grunt.registerTask('commit', ['jshint', 'uglify', 'jasmine:toCommit', 'concat']);

};