/* global UtilDevTools, FileTree, Mousetrap, ShowInDevTools */
function WindowController($scope, ContextRegister, templateManager, ApplicationContext, eventAggregator, DevToolsService, ScreenService, DeveloperTreeService, DTCustomizationService){
	ContextRegister.register('WindowController', this);

	var invalidItems = [];
	var self = this;
	var MainWindow = UtilDevTools.getMainWindow();
	var DevToolsWindow = UtilDevTools.getDevToolsWindow();
	var JS_MODE = 'files';
	var JSON_MODE = 'json';
	var MENU_MODE = 'menu';

	function getDevToolsController(){
		return ApplicationContext.DevToolsController;
	}

	function isDevToolsInSameWindow(){
		return MainWindow.showDevtoolsInSameWindow;
	}

	function prepareMessageAndSend(functionName, args){
		if(DevToolsWindow && DevToolsWindow.sendMessageToDevTools){
			var messageToSend = {
				functionName: functionName,
				args: args
			};
			return DevToolsWindow.sendMessageToDevTools(messageToSend);
		}
	}

	function getInvalidItems(){
		return invalidItems;
	}

	function setErrorsValidationInTree(){
		prepareMessageAndSend("setErrorsValidationInTree", getInvalidItems());
	}

	function reloadErrorsValidationInDevToolsTree(){
		prepareMessageAndSend("reloadErrorsValidationInTree");
	}

	function getRigthComponentPath(componentPath, mode, activeItem){
		var condition = !componentPath && activeItem;
		condition = condition && (mode == 'files' || activeItem.parent == DevToolsWindow.currentContainer.name);
		componentPath = componentPath || [];
		return condition ? activeItem.path : componentPath;
	}

	function enterInDevToolsWindow(componentPath, mode){
		var activeItem = DevToolsService.getLocalActiveItem();
		var cursorPosition = activeItem ? activeItem.cursorPosition : {};
		mode = !mode && activeItem ? activeItem.mode : mode;
		componentPath = getRigthComponentPath(componentPath, mode, activeItem);
		setErrorsValidationInTree();
		if(mode != 'files'){
			reloadErrorsValidationInDevToolsTree();
		}
		prepareMessageAndSend("inspectSentComponent", { componentPath: componentPath , mode: mode, cursorPosition: cursorPosition});
		if(!isDevToolsInSameWindow()){
			window.open("", "DevTools").focus();
		}
	}

	function setUnloadedItems(mode, items){
		prepareMessageAndSend("setUnloadedItems", {mode: mode, items: items, setItems: true});
	}

	function getUnloadedItems(mode){
		return prepareMessageAndSend("getUnloadedItems", mode);
	}

	function removeBackDotsOfPath(path){
		path = path.replace(/\\/g, '/');
		path = path.replace(/\/\//g,'/');
		path = path.replace(/\.\.\//g, '');
		return path;
	}

	function prepareFileMatrix(files){
		var mainScripts = [];
		for(var i in files){
			var src = files[i];
			if(src){
				mainScripts.push({
					"src" : removeBackDotsOfPath(src),
					"relativeUrl": src
				});
			}
		}
		mainScripts.sort(function (scriptA, scriptB){
			var strA = scriptA.src.toLowerCase();
			var strB = scriptB.src.toLowerCase();
			if(strA > strB){
				return 1;
			}
			if(strA < strB){
				return -1;
			}
			return 0;
		});
		return mainScripts;
	}

	function addFiles(files){
		if(files && DevToolsWindow && !DevToolsWindow.treeScript){
			DevToolsWindow.treeScript = null;
			files = prepareFileMatrix(files);
			var treeScript = FileTree.build(files);
			DevToolsWindow.treeScript = treeScript;
			setUnloadedItems(JS_MODE, treeScript);
			if(!getUnloadedItems(JS_MODE)){
				MainWindow.publishDevToolsReady();
			}
		}
	}

	this.updateScriptTree = function(){
		MainWindow.executeFunction('FileManagementService', 'reloadFiles');
	};

	this.setScriptsFiles = UtilDevTools.buildDevToolsWindowMethod(function(scriptsFiles){
		DevToolsService.setFilesToLocalStorage(scriptsFiles);
		addFiles(scriptsFiles);
	});

	function priorToBuildingTree(){
		MainWindow.executeFunction('FileManagementService', 'findFiles');
	}

	function prepareDevToolsWindow(mode){
		var condition = false;
		var activeItem = DevToolsService.getLocalActiveItem();
		mode = !mode && activeItem ? activeItem.mode : mode;
		DevToolsWindow = getDevToolsWindow();
		DevToolsWindow.currentContainer = templateManager.container;
		DeveloperTreeService.setTreeInWindow(MENU_MODE, DevToolsWindow);
		if(mode == JS_MODE){
			condition = DevToolsWindow.treeScript && !getUnloadedItems(mode);
		} else if(DevToolsWindow.currentContainer){
			setUnloadedItems(JSON_MODE, DevToolsWindow.currentContainer);
			condition = !getUnloadedItems(JSON_MODE);
		}
		if(condition){
			MainWindow.publishDevToolsReady();
		}
	}

	function runDeveloperEditMode(args){
		eventAggregator.publish('developerEnableEditMode', args);
	}

	function runDeveloperHideMode(args){
		eventAggregator.publish('developerEnableHideMode', args);
	}

	function triggerResize(owner){
		var event = document.createEvent('Event');
		event.initEvent('resize', true, true);
		owner.dispatchEvent(event);
	}

	function getDevToolsWindow(){
		var devToolsWindow;
		if(isDevToolsInSameWindow()){
			devToolsWindow = window.frames["dev-tools"];
		} else {
			devToolsWindow = window.open(window.location + "?devtools=true", "DevTools", "width=" + window.screen.width + ", height=" + window.screen.height);
		}
		return devToolsWindow;
	}

	this.updateActiveItemInEditor =  function(paths){
		prepareMessageAndSend("updateActiveItemInEditor", paths);
	};
	this.setFilesToLocalStorage = function(scriptsFiles){
		MainWindow.setFilesToLocalStorage(scriptsFiles);
	};

	this.addInvalidItem =  function(invalidItem){
		invalidItems.push(invalidItem);
		prepareMessageAndSend("addErrorValidationInTree", invalidItem);
	};

	this.clearInvalidItems = function clearInvalidItems(){
		invalidItems = [];
		prepareMessageAndSend("clearErrorsValidationInTree");
	};

	this.isMainWindow = function(){
		return window === MainWindow;
	};

	this.isDevToolsWindow = function(){
		return window === DevToolsWindow;
	};

	this.isDevToolsOpened = function(){
		return !!DevToolsWindow;
	};

	this.reloadMainWindow =  function(){
		MainWindow.location.reload(true);
	};

	function closeDevToolsWindow(){
		MainWindow.closeDevToolsWindow();
	}

	function openDevToolsWindow(componentPath, mode){
		if(DTCustomizationService.getAccessEditMode()){
			var evt = eventAggregator.subscribe('devToolsReady', function(){
				enterInDevToolsWindow(componentPath, mode);
				eventAggregator.unSubscribe('devToolsReady', evt);
			});
			var files = DevToolsService.getFilesFromLocalStorage();
			prepareDevToolsWindow(mode);
			MainWindow.devtools = true;
			if(!files || !files.length){
				priorToBuildingTree();
			} else {
				addFiles(files);
			}
			DevToolsService.setLocalDevToolsFlag(MainWindow.devtools);
			templateManager.updateTemplate();
			triggerResize(MainWindow);
		}
	}

	function reopenDevToolsWindow(){
		closeDevToolsWindow();
		openDevToolsWindow();
		if(MainWindow.showDevtoolsInSameWindow){
			MainWindow.publishDevToolsReady();
		}
	}

	this.sendToDevTools = UtilDevTools.buildMainWindowMethod(function(componentPath){
		if(self.isDevToolsOpened()){
			enterInDevToolsWindow(componentPath);
		} else {
			openDevToolsWindow(componentPath);
		}
		window.inspection = false;
	});

	this.showNotificationInDevTools = function(message, type){
		if(self.isMainWindow() && self.isDevToolsOpened()){
			var args = {
				message: message,
				type: type
			};
			prepareMessageAndSend("showNotificationInDevTools", args);
		} else {
			ScreenService.notificationMessage(message, type);
		}
	};

	this.showNotificationInMainWindow = function(message, type){
		if(self.isMainWindow()){
			ScreenService.notificationMessage(message, type);
		} else {
			MainWindow.showNotificationInMainWindow(message, type);
		}
	};

	// Binds keyboard shortcut to open/close devtools
	Mousetrap.bind('mod+f12', function(){
		if(self.isMainWindow() && !DevToolsWindow){
			openDevToolsWindow();
		} else {
			closeDevToolsWindow();
		}
	});

	if(self.isMainWindow()){
		$scope.$watch(function(){
			return templateManager.container;
		}, function(){
			if(DevToolsWindow){
				DevToolsWindow.currentContainer = templateManager.container;
				ShowInDevTools.setTree(DevToolsWindow.currentContainer.__metadata);
				setUnloadedItems(JSON_MODE, DevToolsWindow.currentContainer);
				self.clearInvalidItems();
			} else if(templateManager.container && DevToolsService.getLocalDevToolsFlag()){
				openDevToolsWindow();
			}
		});

		MainWindow.executeFunction = function(instanceName, functionName, args){
			var instance = ApplicationContext[instanceName];
			if(instance){
				return instance[functionName].apply(this, args);
			}
		};
		MainWindow.getDevToolsWindow = function(){
			return DevToolsWindow;
		};
		MainWindow.showConfirmMessageInMainWindow = function(txt, confirmFunction, failFunction){
			ScreenService.confirmMessage(txt, 'confirmation', confirmFunction, failFunction);
		};
		MainWindow.showDialogInMainWindow = function(message, actions){
			ScreenService.showCustomDialog(message, 'confirmation', null, actions);
		};
		MainWindow.showNotificationInMainWindow = function(message, type){
			ScreenService.notificationMessage(message, type);
		};
		MainWindow.onbeforeunload = function(){
			if(self.isDevToolsOpened()){
				MainWindow.closeDevToolsByMainWindow = true;
				DevToolsWindow.close();
			}
		};
		MainWindow.setFilesToLocalStorage = function(scriptsFiles){
			DevToolsService.setFilesToLocalStorage(scriptsFiles);
		};
		MainWindow.publishDevToolsReady = function(){
			eventAggregator.publish('devToolsReady');
		};
		MainWindow.closeDevToolsWindow = function(){
			if(DevToolsWindow && !isDevToolsInSameWindow()){
				DevToolsWindow.close();
			}
			prepareMessageAndSend("clearActiveItem");
			MainWindow.focus();
			MainWindow.devtools = false;
			DevToolsService.setLocalDevToolsFlag(MainWindow.devtools);
			DevToolsWindow = null;
			templateManager.updateTemplate();
			triggerResize(MainWindow);
		};
		MainWindow.changeWindowMode = function(){
			MainWindow.showDevtoolsInSameWindow = !MainWindow.showDevtoolsInSameWindow;
			reopenDevToolsWindow();
		};
	} else {
		DeveloperTreeService.updateTree(MENU_MODE);
		DevToolsWindow.sendMessageToDevTools = function(message){
			templateManager.updateTemplate();
			var devToolsController = getDevToolsController();
			if(devToolsController){
				return devToolsController.receiveMessages(message);
			}
		};
		DevToolsWindow.publishFinishedRenderTreeItems = function(){
			eventAggregator.publish('finishedRenderTreeItems');
		};
		DevToolsWindow.onbeforeunload = function(){
			if(!MainWindow.closeDevToolsByMainWindow){
				closeDevToolsWindow();
			}
		};
	}

	var auxValue;
	Object.defineProperty(window, 'inspection', {
		set: function(newValue){
			auxValue = newValue;
			if(self.isMainWindow() && DevToolsWindow){
				if(DevToolsWindow.inspection !== newValue){
					DevToolsWindow.inspection = newValue;
				} else {
					runDeveloperEditMode({
						'enableEditMode': newValue
					});
				}
			} else if(self.isDevToolsWindow() && MainWindow.inspection !== newValue){
				MainWindow.inspection = newValue;
			}
			templateManager.updateTemplate();
		},
		get: function(){
			return auxValue;
		}
	});
	var auxHideValue;
	Object.defineProperty(window, 'hide', {
		set: function(newValue){
			auxHideValue = newValue;
			runDeveloperHideMode(newValue);
		},
		get: function(){
			return auxHideValue;
		}
	});
}
