/* global UtilDevTools */
function DeveloperService(restEngine, localStorageEngine, templateManager){

    var DEV_CLIENT = "developerTools";// default devClient
    var DATE_TO_EXPIRE = 3600000;
    var CONFIG_FILE = '.developerConfig.json';
    var LOCALVAR_DSLOGIN = 'ds_login',
        LOCALVAR_FILES = 'devToolsScriptsFiles',
        LOCALVAR_DEVTOOLS = 'devTools',
        LOCALVAR_CLIENT_EMAIL = 'ds_clientEmail',
        LOCALVAR_ACTIVE_ITEM = 'devToolsActiveItem',
        LOCALVAR_PROJECT_OBJECT = 'ds_projectObject';

    this.getProject = function(){
        return localStorageEngine.getLocalVar(LOCALVAR_PROJECT_OBJECT);
    };

    this.setProject = function(){
        var project = {
            "id": templateManager.project.id,
            "name": templateManager.project.label
        };
        localStorageEngine.setLocalVar(LOCALVAR_PROJECT_OBJECT, project);
    };

    this.setLocalDevToolsFlag = function(value){
        localStorageEngine.setLocalVar(LOCALVAR_DEVTOOLS, !!value);
    };

    this.setLocalActiveItem = function(itemName, path, mode, parent, cursorPosition){
        parent = parent || "";
        localStorageEngine.setLocalVar(LOCALVAR_ACTIVE_ITEM, {
            mode : mode,
            path : path,
            parent: parent,
            cursorPosition: cursorPosition || {},
            name : itemName
        });
    };

    this.setLocalEmail = function(email){
        localStorageEngine.setLocalVar(LOCALVAR_CLIENT_EMAIL, email);
    };

    this.getLocalEmail = function(){
        return localStorageEngine.getLocalVar(LOCALVAR_CLIENT_EMAIL);
    };

    this.getLocalDevToolsFlag = function(){
        return localStorageEngine.getLocalVar(LOCALVAR_DEVTOOLS);
    };

    this.getLocalActiveItem = function(){
        return localStorageEngine.getLocalVar(LOCALVAR_ACTIVE_ITEM);
    };

    this.clearActiveItem = function(){
        localStorageEngine.remove(LOCALVAR_ACTIVE_ITEM);
    };

    var setShowDevtoolsInSameWindow = function(){
        var MainWindow = UtilDevTools.getMainWindow();
        if(typeof MainWindow.showDevtoolsInSameWindow == 'undefined'){
            MainWindow.showDevtoolsInSameWindow = templateManager.project.showDevtoolsInSameWindow;
        }
    };

    var normalizeInCustomizationMode = function(){
        var inCustomizationMode = window.inCustomizationMode;
        if(typeof inCustomizationMode == 'undefined'){
            inCustomizationMode = templateManager.project.inCustomizationMode;
            window.inCustomizationMode = inCustomizationMode;
        } else {
            templateManager.project.inCustomizationMode = inCustomizationMode;
        }
        templateManager.inCustomizationMode = inCustomizationMode;
        return inCustomizationMode;
    };

    this.inDeveloperModePromise = templateManager.projectPromise.then(function(){
        setShowDevtoolsInSameWindow();
        var responseObj = {};
        var inCustomizationMode = normalizeInCustomizationMode();
        var inDeveloperMode = inCustomizationMode;
        if(!inDeveloperMode){
            var defineInProject = typeof templateManager.inDeveloperMode == 'undefined';
            inDeveloperMode = defineInProject ? templateManager.project.inDeveloperMode : templateManager.inDeveloperMode;
        }
        window.inDeveloperMode = inDeveloperMode;
        templateManager.inDeveloperMode = inDeveloperMode;
        templateManager.project.inDeveloperMode = inDeveloperMode;
        responseObj.inDeveloperMode = inDeveloperMode;
        responseObj.inCustomizationMode = inCustomizationMode;
        return responseObj;
    });

    this.requestMetadataDeveloper = function(path){
        return restEngine.requestMetaData(path).then(function(data){
            return data;
        }).catch(function(){
            return restEngine.requestMetaData(path, false, false, './').then(function(data){
                return data;
            });
        });
    };

    this.clearLocalDevToolsFlag = function(){
        localStorageEngine.remove(LOCALVAR_DEVTOOLS);
    };

    this.setFilesToLocalStorage = function(scriptsFiles){
        if(scriptsFiles){
            localStorageEngine.setLocalVar(LOCALVAR_FILES, {"files": scriptsFiles, "date": new Date()});
        }
    };

    this.getFilesFromLocalStorage = function(){
        var files = [];
        var obj = localStorageEngine.getLocalVar(LOCALVAR_FILES);
        if(obj && obj.date && obj.files){
            var lastDate = new Date(obj.date);
            var newDate = new Date();
            if((newDate - lastDate) < DATE_TO_EXPIRE){
                files = obj.files;
            }
        }
        return files;
    };
}
